######################################################################
## Global variables and configuration parameters
######################################################################

## --------------------------------------------------------------------
## Appearance configuration
## --------------------------------------------------------------------

## generic font type definition
set ::fontType_std {Helvetica}
set ::fontType_fix {Courier}
set ::fontSize_std {10}
set ::fontSize_big {12}
set ::fontSize_min {9}
## combined font type definition
set ::font      "$::fontType_std $::fontSize_std"
set ::font_big  "$::fontType_std $::fontSize_big"
set ::font_min  "$::fontType_std $::fontSize_min"

option add *font                        "$::fontType_std $::fontSize_std"
option add *titleFont                   "$::fontType_std $::fontSize_std bold"
option add *entryFont                   "$::fontType_std $::fontSize_std"
option add *textFont                    "$::fontType_std $::fontSize_std"
option add *unitFont                    "$::fontType_std $::fontSize_std"
option add *labelFont                   "$::fontType_std $::fontSize_std"
option add *msgFont                     "$::fontType_std $::fontSize_big"

option add *LoggingFrame.timeFont       "$::fontType_fix $::fontSize_min"
option add *LoggingFrame.logFont        "$::fontType_std $::fontSize_std"

option add *ScienceEntry.flashing yes
option add *EntryFrame.scrollRelief flat
option add *Tabnotebook.bevelAmount 4 startupFile
option add *EntryFrame.tabPos w
if {$::tcl_platform(platform) == {unix}} {
    option add *sbWidth 10
}

option add *activeBackground           grey70    ;# the hightlight background color if mouse is over an item

## --------------------------------------------------------------------
## Platform specific
## --------------------------------------------------------------------
## Temp dir
if {[info exists ::env(TEMP)]} {
    set ::TEMP_PATH [file normalize $::env(TEMP)]
} elseif {[info exists ::env(TMP)]} {
    set ::TEMP_PATH [file normalize $::env(TMP)]
} else {
    set ::TEMP_PATH $::SCOE_PATH
}

## Spool dir, where hardcopy files will be saved
if {[info exists ::env(SPOOL)]} {
    set ::SPOOL_PATH [file normalize $::env(SPOOL)]
} else {
    if {[info exists ::env(HOME)] && [file isdirectory [file join $::env(HOME) spool]]} {
        set ::SPOOL_PATH [file join $::env(HOME) spool]
    } elseif {[file isdirectory [file join $::TEMP_PATH spool]]} {
        set ::SPOOL_PATH [file join $::TEMP_PATH spool]
    } else {
        set ::SPOOL_PATH $::TEMP_PATH
    }
}

## IMAGEMAGICK binary, for converting images
if {[info exists ::env(MAGICK_HOME)]} {
    if {$tcl_platform(platform) == {windows}} {
        set ::IMAGEMAGICK_BIN [file join $::env(MAGICK_HOME) convert.exe]
    } else {
        set ::IMAGEMAGICK_BIN [file join $::env(MAGICK_HOME) convert]
    }
} else {
    if {$tcl_platform(platform) == {windows}} {
        set ::IMAGEMAGICK_BIN "convert.exe"
    } else {
        set ::IMAGEMAGICK_BIN [file join usr bin convert]
    }
}
set ::IMAGEMAGICK_PARAMETER_A4 "-scale 50% -rotate -90 -page 565x820+15+20"
# -negate

## Printer command
proc printCommand {args} {
    set options(-filename) {}
    array set options $args
    set options(-filename) [file nativename $options(-filename)]
    if {$::tcl_platform(platform) == {windows}} {
        set msg [exec print /D:\\\\vies154a\\hp_3147a $options(-filename)]
        if {[string first "being printed" $msg] < 0} {
            error "printCommand: $msg"
        }
    } else {
        if {[catch {exec lpr $options(-filename)} msg]} {
            error "printCommand: $msg"
        }
    }
}

set ::archive_directory $::SPOOL_PATH

## --------------------------------------------------------------------
## Database configuration
## --------------------------------------------------------------------
set ::SCOE_CONFIG(database.Name) scoedb_gal-ttc
set ::SCOE_CONFIG(database.Host) localhost
set ::SCOE_CONFIG(database.User) scoe
set ::SCOE_CONFIG(database.Password) scoe

## --------------------------------------------------------------------
## Reporting
## --------------------------------------------------------------------
## Report modes {None File Display Print Print/Display}
set ::SCOE_CONFIG(report.ActiveMode) "Display"

## --------------------------------------------------------------------
## Logging
## --------------------------------------------------------------------
## startup log levels {zdebug debug trace note result c&c warning error}
set ::SCOE_CONFIG(log.DispLevel) trace
set ::SCOE_CONFIG(log.FileLevel) debug
## archiving
set ::SCOE_CONFIG(log.HKFileName) "hktm_${::SCOE}_"
set ::SCOE_CONFIG(log.FileName) "log_${::SCOE}_"
set ::SCOE_CONFIG(log.FilePath) $::SPOOL_PATH
set ::SCOE_CONFIG(log.MaxFileSize) 1e6
set ::SCOE_CONFIG(log.SplitByDate) yes
set ::SCOE_CONFIG(log.SplitDateInDir) yes
## time formats {-withdate -withtime -withms}
set ::SCOE_CONFIG(log.GMT) yes
set ::SCOE_CONFIG(log.FileTimeArgs) "-withdate -withtime -withms"
set ::SCOE_CONFIG(log.DispTimeArgs) "-withtime -withms"
## displaying
set ::SCOE_CONFIG(log.KeepMaxLines) 500

## --------------------------------------------------------------------
## EGSE
## --------------------------------------------------------------------
set ::SCOE_CONFIG(egse.CmdFinPackets) {CMDFIN}                  ;# command completion packets
set ::SCOE_CONFIG(egse.ReplyAfterSet) no                        ;# send reply after each set command
set ::SCOE_CONFIG(egse.AutoRemote) no                           ;# after EGSE socket connection switch to REMOTE
set ::SCOE_CONFIG(egse.Protocol) eden_ifscoe                     ;# egse protocol used
set ::SCOE_CONFIG(egse.PortFrom) 6000                           ;# port from egse
set ::SCOE_CONFIG(egse.PortTo) 6000                             ;# port to egse !!! not used for HP CCS

## --------------------------------------------------------------------
## Software configuration
## --------------------------------------------------------------------
## startup behaviour
set ::SCOE_CONFIG(hktm.CycleAfterStartup) {SAFETY}

## --------------------------------------------------------------------
## Windows
## --------------------------------------------------------------------
set ::SCOE_CONFIG(windows.Icon) {}                              ;# (runtime) the main icon used as default
set ::SCOE_CONFIG(windows.OpOK) {}                              ;# (runtime) the one and only operator ok window
set ::SCOE_CONFIG(windows.CCMain) {}                            ;# (runtime) the main command overview window
set ::SCOE_CONFIG(windows.CCSeq) {}                             ;# (runtime) the command sequence window
set ::SCOE_CONFIG(windows.Plot) {}                              ;# (runtime) the first initial plot window

## --------------------------------------------------------------------
## Screen information - there is no way to find the number of screens and other parameters ...
## --------------------------------------------------------------------
set ::SCOE_CONFIG(screen.Width) {}                              ;# (runtime) the main icon used as default
set ::SCOE_CONFIG(screen.Height) {}                             ;# (runtime) the one and only operator ok window

## --------------------------------------------------------------------
## SCOE specific
## --------------------------------------------------------------------
set ::SCOE_CONFIG(scoe.SecondInstCheckPort) 4242                ;# port number to open so we can easily check if scoe is started in seconds instance
set ::SCOE_CONFIG(scoe.Debug) no                                ;# startup in debug mode is an command line option
set ::SCOE_CONFIG(scoe.PayloadModel) {}                         ;# (command-line) the spacecraft we are connected
## objects
set ::SCOE_CONFIG(scoe.CmdIfObj) {}                             ;# command interface object
set ::SCOE_CONFIG(scoe.NObIfObj) {}                             ;# nobject handler
set ::SCOE_CONFIG(scoe.DbIfObj) {}                              ;# (runtime) DB Server instance
set ::SCOE_CONFIG(scoe.ResultIfObj) {}                          ;# (runtime) Result Handler
set ::SCOE_CONFIG(scoe.HKTMArchObj) {}                          ;# HKTM archive object
set ::SCOE_CONFIG(scoe.Scheduler) {}                            ;# Scheduler object
set ::SCOE_CONFIG(scoe.LogIfObj) {}                             ;# (runtime) Logging object

## --------------------------------------------------------------------
## Hardware configuration
## --------------------------------------------------------------------
## INTernal or EXTernal reference oscillator
set ::SCOE_CONFIG(device.ReferenceOscillator) EXT

## --------------------------------------------------------------------
## IP/LAN configuration, GPIB configuration
## --------------------------------------------------------------------
## hosts
set ::SCOE_CONFIG(host.X.GpibGw) xgpibgw
set ::SCOE_CONFIG(host.S.GpibGw) sgpibgw
## GPIB interface string
set ::SCOE_CONFIG(gpib.X.IfStr) "lan\\\[$::SCOE_CONFIG(host.X.GpibGw)\\\]:gpib0"
set ::SCOE_CONFIG(gpib.S.IfStr) "lan\\\[$::SCOE_CONFIG(host.S.GpibGw)\\\]:gpib0"

## --------------------------------------------------------------------
## SCOE information
## --------------------------------------------------------------------
set ::SCOE_CONFIG(info.Version) {}                              ;# this information ist taken from the version.tcl file in the specific scoe directory
set ::SCOE_CONFIG(info.VersionDate) {}                          ;# this information ist taken from the version.tcl file in the specific scoe directory
set ::SCOE_CONFIG(info.Hotline) {}                              ;# this information ist taken from the version.tcl file in the specific scoe directory
set ::SCOE_CONFIG(info.WebSite) {}                              ;# this information ist taken from the version.tcl file in the specific scoe directory

## --------------------------------------------------------------------
## startup parameters from command line
## --------------------------------------------------------------------
set ::SCOE_CONFIG(startup.Debug) no                             ;# do more debug logging
set ::SCOE_CONFIG(startup.Simulated) no                         ;# run hardware in simulated mode
set ::SCOE_CONFIG(startup.Startup) normal                       ;# do normal or hot startup
set ::SCOE_CONFIG(startup.Payload) $::SCOE                      ;# the initial payload

## --------------------------------------------------------------------
## Error values for exec_check and exec_eval
## --------------------------------------------------------------------
array set ::CCERROR_VALUES { "noCommand"              0
                             "commandRunning"         1
                             "commandSuccess"         2
                             "commandAborted"         3
                             "commandError"           4
                             "parameterError"         5 }

## --------------------------------------------------------------------
## Calibration factors
## --------------------------------------------------------------------

## --------------------------------------------------------------------
## Synoptic configuration
## --------------------------------------------------------------------

## Settings for Synoptic window
global synconf
set synconf(tkobjfile,.top${::SCOE}_Synoptic) tcl/scoe/$::SCOE/gui/synoptic.tkobj
set synconf(title,.top${::SCOE}_Synoptic) "Synoptic $::SCOE SCOE"
set synconf(geometry,.top${::SCOE}_Synoptic) 880x580+150+340
#set synconf(simulated,.top${::SCOE}_Synoptic) FULL
set synconf(simulated,.top${::SCOE}_Synoptic) NO

######################################################################
## End of configurable items
######################################################################

## This procedure is called at startup to source this config file
proc InitConfig {} {}








