########################
## TRACEING

proc traceEGSEconnection {var index op} {
    ## !! for safety reasons catch all procedures !!
    ## if you dont - and something will go wrong - the scoe state machine might go bla bla

    ## trace the windows that are inherited by GenericWindow
    if {[catch traceAutoSwitchToRemote msg]} {
        Log e "traceEGSEconnection: While executing traceAutoSwitchToRemote: $msg"
    }

    ## trace all generic buttons
    if {[catch traceButtonState msg]} {
        Log e "traceEGSEconnection: While executing traceButtonState: $msg"
    }
}

proc traceSCOEactivity {var index op} {
    ## !! for safety reasons catch all procedures !!
    ## if you dont - and something will go wrong - the scoe state machine might go bla bla

    ## trace all generic buttons
    if {[catch traceButtonState msg]} {
        Log e "traceSCOEactivity: While executing traceButtonState: $msg"
    }

    ## trace lockout state of devices
    if {[catch traceLocalLockoutState msg]} {
	Log e "traceSCOEactivity: While executing traceLocalLockoutState: $msg"
    }

    ## trace synoptic state
    if {[catch traceSynopticLockoutState msg]} {
	Log e "traceSCOEstate: While executing traceSynopticLockoutState: $msg"
    }
}

proc traceSCOEstate {var index op} {
    ## !! for safety reasons catch all procedures !!
    ## if you dont - and something will go wrong - the scoe state machine might go bla bla

    ## trace the windows that are inherited by GenericWindow
    if {[catch traceNObjectState msg]} {
        Log e "traceSCOEstate: While executing traceNObjectState: $msg"
    }

    ## trace all generic buttons
    if {[catch traceButtonState msg]} {
        Log e "traceSCOEstate: While executing traceButtonState: $msg"
    }

    ## trace lockout state of devices
    if {[catch traceLocalLockoutState msg]} {
	Log e "traceSCOEstate: While executing traceLocalLockoutState: $msg"
    }
    
    ## trace synoptic state
    if {[catch traceSynopticLockoutState msg]} {
	Log e "traceSCOEstate: While executing traceSynopticLockoutState: $msg"
    }
}

## TRACEING
########################
########################
## TRACEING HELPERS

proc traceAutoSwitchToRemote {} {
    if {($::EGSE_connection == "CONNECTED") && ($::SCOE_state == "LOCAL") && ([valbool $::SCOE_CONFIG(egse.AutoRemote)])} {
        set ::SCOE_state "REMOTE"
        scoe_server send_cmd "TRANSFER" "REMOTE"
    }
}

proc traceSynopticLockoutState {} {
    if {![winfo exists .topSynoptic]} {return}
    if {($::SCOE_state == "REMOTE") || ($::SCOE_activity == "RUNNING")} {
        .topSynoptic.c itemconfigure all -state disabled
    } else {
        .topSynoptic.c itemconfigure all -state normal
    }
}

proc traceNObjectState {{nobjects {}}} {
    if {$nobjects == {}} {set nobjects [find objects ::* -isa GNObject]}

    if {$::SCOE_state == "MAINTENANCE"} {
        foreach obj $nobjects {
            after idle [list $obj configure -maintenanceMode yes]
        }
    } else {
        foreach obj $nobjects {
            after idle [list $obj configure -maintenanceMode no]
        }
    }
}

proc traceButtonState {} {
    if {![array exists ::disabled_states]} return

    foreach wcmd [array names ::disabled_states] {
        ## extract the parts of the command
        foreach {type widget part} [split $wcmd !] {}
        ## evaluate the state
        set evalState [string map {1 "disabled" 0 "normal"} [expr $::disabled_states($wcmd)]]
        ## check type
        switch $type {
            "widget" {
                set cmd [list $widget configure -state $evalState]
            }
            "menu" {
                set cmd [list $widget menuconfigure $part -state $evalState]
            }
            default {
                error "traceButtonState: Unknown type '$type'"
            }
        }
        ## if the command fails we remove it from array - it might be removed from user level
        if {[catch $cmd msg]} {
            Log d "traceButtonState: $msg" -forcefile yes
            unset ::disabled_states($wcmd)
        }
    }
}

## TRACEING HELPERS
########################

proc exeUserPrivileges {} {
    if {![array exists ::disabled_level]} return

    foreach wcmd [array names ::disabled_level] {
        foreach {type widget part} [split $wcmd !] {}
        ## check type
        switch $type {
            "widget" {
                #set cmd [list $widget configure -state $evalState]
            }
            "menu" {
                set cmd [list $widget delete $part]
            }
            default {
                error "exeUserPrivileges: Unknown type '$type'"
            }
        }
        ## evaluate the state
        if {$::scoe_operator_type < $::disabled_level($wcmd)} {set cmd {}}
        ## it might fail
        if {[catch $cmd msg]} {
            Log d "exeUserPrivileges: $msg" -forcefile yes
            unset ::disabled_level($wcmd)
        }
    }
}

proc InitLogging {{parent {}}} {
    #### restore default TclTk coulors
    set logging_widget [LoggingFrame $parent.wLogging -visibleitems 0x0 \
                            -gmt $::SCOE_CONFIG(log.GMT) \
                            -dispTimeArgs $::SCOE_CONFIG(log.DispTimeArgs) \
                            -fileTimeArgs $::SCOE_CONFIG(log.FileTimeArgs) \
                            -isCollapsible yes]

    $logging_widget addLevel -char D -name "Debug"   -color gray95
    $logging_widget addLevel -char T -name "Trace"   -color gray85
    $logging_widget addLevel -char N -name "Note"    -color lightblue
    $logging_widget addLevel -char R -name "Result"  -color lightblue4
    $logging_widget addLevel -char C -name "C&C"     -color green -instatus yes
    $logging_widget addLevel -char W -name "Warning" -color yellow -instatus yes
    $logging_widget addLevel -char H -name "High"    -color orange -inmenu no -instatus yes
    $logging_widget addLevel -char E -name "Error"   -color IndianRed1 -instatus yes

    pack $logging_widget -anchor nw -fill both -expand y

    $logging_widget configure -dispLogLevel [string toupper [string index $::SCOE_CONFIG(log.DispLevel) 0]]
    $logging_widget configure -fileLogLevel [string toupper [string index $::SCOE_CONFIG(log.FileLevel) 0]]

    $logging_widget configure \
        -fileName $::SCOE_CONFIG(log.FileName) \
        -filePath $::SCOE_CONFIG(log.FilePath) \
        -maxFileSize $::SCOE_CONFIG(log.MaxFileSize) \
        -splitByDate $::SCOE_CONFIG(log.SplitByDate) \
        -splitDateInDir $::SCOE_CONFIG(log.SplitDateInDir) \
        -maxLines $::SCOE_CONFIG(log.KeepMaxLines)

    ### make the widget available for logging
    set ::SCOE_CONFIG(scoe.LogIfObj) $logging_widget

    global bgerror_file
    set bgerror_file {}
}

proc InitStatus {pf} {
    set row1 {}
    set row2 {}

    ## SCOE state
    set ::SCOE_state "init"
    gen_statefield $pf.state "SCOE state:" ::SCOE_state {
        { "INIT"        white       "INIT"             }
        { "LOCAL"       lightblue   "LOCAL"            }
        { "MANUAL"      yellow      "MANUAL"           }
        { "MAINTENANCE" orange      "MAINTENANCE"      }
        { "REMOTE"      green       "REMOTE"           }
    }
    lappend row1 $pf.state

    ## SCOE activity
    set ::SCOE_activity "IDLE"
    gen_statefield $pf.activity "SCOE activity:" ::SCOE_activity {
        { "IDLE"    lightblue   "IDLE"    }
        { "RUNNING" yellow      "RUNNING" }
    }
    lappend row1 $pf.activity

    ## EGSE connection
    set ::EGSE_connection "DOWN"
    gen_statefield $pf.connection "Socket:" ::EGSE_connection {
        { "DOWN"        orange  "DOWN"      }
        { "ACCEPTING"   yellow  "ACCEPTING" }
        { "CONNECTED"   green   "CONNECTED" }
    }
    lappend row1 $pf.connection

    ## Selftest state
    set ::SCOE_selftest "UNKNOWN"
    gen_statefield $pf.selftest "Selftest state:" ::SCOE_selftest {
        { "UNKNOWN"    white    "UNKNOWN"}
        { "PASSED"     green    "PASSED"}
        { "FAILED"     red      "FAILED"}
        { "OVERRIDE"   orange   "OVERRIDE"}
    }
    lappend row1 $pf.selftest

    # ONLINE/OFFLINE
    set ::SCOE_uplink "OFFLINE"
    gen_statefield $pf.uplink "S/C interface:" ::SCOE_uplink {
        { "OFFLINE" lightblue   "OFFLINE" }
        { "ONLINE"  yellow      "ONLINE"  }
    }
    lappend row1 $pf.uplink

    ## Switch Local
    button $pf.local -text "Switch LOCAL" -command SwitchLocal
    set ::disabled_states(widget!$pf.local) {$::SCOE_state == "LOCAL"}
    lappend row2 $pf.local

    ## Abort test script
    button $pf.abort -text "Abort command:" -command {testSequenceAbort} -font $::font_min
    set ::disabled_states(widget!$pf.abort) {$::SCOE_activity == "IDLE"}
    lappend row2 $pf.abort

    ## Running test sequence
    set ::current_testSequence ""
    label $pf.testScript -textvariable ::current_testSequence -font $::font_min
    lappend row2 $pf.testScript

    ## the sequence gets the rest of the display
    set rowDiff [expr [llength $row1] - [llength $row2]]
    for {set i 1} {$i <= $rowDiff} {incr i} {lappend row2 -}

    ## Progress bar
    RFeedback $pf.progress
    set ::FEEDBACK_WIDGET $pf.progress
    lappend row3 $pf.progress
    for {set i 1} {$i < [llength $row1]} {incr i} {lappend row3 -}

    ## grid the fields
    foreach row [list $row1 $row2 $row3] {
        eval grid $row -sticky nwse -pady 1
    }
    for {set col 0} {$col < [llength $row1]} {incr col} {
        grid columnconfigure $pf $col -weight 1
    }
}

proc SwitchLocal {} {
    if {$::SCOE_state == "REMOTE"} {
        if {[operatorOk "Switch to LOCAL operation.\n\nPress OK to continue or Cancel to abort." -type warning -imageFile "warning_finger.gif" -forceLocal yes -autoReply no]} {
            if {$::EGSE_connection == "CONNECTED"} {
                scoe_server send_cmd "TRANSFER" "LOCAL"
            }
            scoe_server exec_eval -command_name TRANSFER -check_result {mode LOCAL}
        }
    } else {
        set ::SCOE_state "LOCAL"
    }
}

proc LogOut {} {
    if {[operatorOk "Logout confirmation.\n\nPress OK to exit or Cancel to continue working." -type question -imageFile "info_key.gif" -forceLocal yes -autoReply no]} {

        ## note shutting down
        Log w "Shutting down $::SCOE software ..."
        
        ## shut down objects
        foreach configId {scoe.Scheduler scoe.CmdIfObj scoe.NObIfObj} {
            set object $::SCOE_CONFIG($configId)
            set ::SCOE_CONFIG($configId) {}
            catch {delete object $object}
            after 10
            update
        }
        
        ## fini
        after 100
        exit
    } else {
        Log w "Logout cancelled"
    }
}

proc Log {level text args} {
    if {$::SCOE_CONFIG(scoe.LogIfObj) != {}} {
        eval [list $::SCOE_CONFIG(scoe.LogIfObj) add $level $text] $args
    } else {
        puts $text
    }

    if {[find object ::scoe_server] != {}} {
        scoe_server Log_EGSE $level $text
    }
}

proc InitAppWindows {} {
    set fbid [fb new 3]

    ## the main command overview window
    set ::SCOE_CONFIG(windows.CCMain) [CCMainWindow ".#auto"]
    $::SCOE_CONFIG(windows.CCMain) configure -cmdIfObj $::SCOE_CONFIG(scoe.CmdIfObj)
    $::SCOE_CONFIG(windows.CCMain) configure -width 400 -height 260
    fb step $fbid

    ## the command sequence window
    set ::SCOE_CONFIG(windows.CCSeq) [CCSeqWindow ".#auto"]
    $::SCOE_CONFIG(windows.CCSeq) configure -ccMainWindow $::SCOE_CONFIG(windows.CCMain) -cmdIfObj $::SCOE_CONFIG(scoe.CmdIfObj)
    $::SCOE_CONFIG(windows.CCSeq) configure -disabledStateEval {($::SCOE_state == "REMOTE") || ($::SCOE_activity == "RUNNING")}
    $::SCOE_CONFIG(windows.CCSeq) configure -width 600 -height 300
    fb step $fbid

    ## the first main plot window
    set ::SCOE_CONFIG(windows.Plot) [PlotWindow ".#auto"]
    $::SCOE_CONFIG(windows.Plot) configure -width 600 -height 300
    fb step $fbid
}

set ::ALIVETICK_progressSymbols { "..   " " ..  " "  .. " "   .." "  .. " " ..  " }
set ::ALIVETICK_progressSymbolPos 0
proc AliveTick {} {
    if {$::ALIVETICK_progressSymbolPos >= [llength $::ALIVETICK_progressSymbols]} {set ::ALIVETICK_progressSymbolPos 0}
    wm title . "${::SCOE} SCOE [lindex $::ALIVETICK_progressSymbols $::ALIVETICK_progressSymbolPos]"
    incr ::ALIVETICK_progressSymbolPos
    update idletask
}

proc showAboutWindow {} {
    set msg "[$::SCOE_CONFIG(scoe.CmdIfObj) cget -loggingTitle] - (c) by SIEMENS\n\n"
    append msg "\n"
    append msg "SW Version $::SCOE_CONFIG(info.Version), Date $::SCOE_CONFIG(info.VersionDate)\n"
    append msg "TCL $::tcl_patchLevel, tclUtil [package present tclUtil], pgTcl [package present pgTcl]\n"
    append msg "\n\n"
    if {$::SCOE_CONFIG(info.Hotline) != {}} {
        append msg "Hotline: $::SCOE_CONFIG(info.Hotline)"
    }
    if {$::SCOE_CONFIG(info.WebSite) != {}} {
        append msg "WebSite: $::SCOE_CONFIG(info.WebSite)"
    }
    operatorOk $msg -forceLocal yes -symbolFile $::SCOE_PATH/pictures/logo/scoe_mainlogo_full_200h_grey_border.gif
    #$::SCOE_CONFIG(windows.OpOK) configure -msgfont $::font_min
}

proc showSynopticWindow {} {
    if {![winfo exists .topSynoptic]} {
        toplevel .topSynoptic
        wm title .topSynoptic "SYNOPTIC $::SCOE"
        wm protocol .topSynoptic WM_DELETE_WINDOW "wm withdraw .topSynoptic"
        
        ::iwidgets::scrolledcanvas .topSynoptic.c -width 1000 -height 750 \
            -vscrollmode dynamic -hscrollmode dynamic -autoresize yes -automargin 0
        pack .topSynoptic.c -padx 10 -pady 10 -fill both -expand yes
        .topSynoptic.c xview moveto 0
        .topSynoptic.c yview moveto 0
        
        set syn_canvas .topSynoptic.c
        source [file join $::CONFIG_PATH synoptic.tkobj]

        foreach obj [scoe_synoptic_objs] {
            if {[catch {$obj configure -synoptic_canvas .topSynoptic.c -synoptic_pos $syn_position($obj); $obj update_synoptic} msg]} {
                Log e "SYNOPTIC: Attaching object '$obj': $msg"
                Log d $::errorInfo
            }
        }
        traceSynopticLockoutState
    } else {
        wm deiconify .topSynoptic
    }
}
