
#%ProcedureHeader################################################################
#Name        : extractCompArgs
#
#Author      : Thomas Perschak 2004/01/07
#modified by : <none>
#Abstract    : Find components in args
#Arguments   : syntax defines how the parameters can be arranged comp-comp or comp-val and opt-opt or opt-val
#            : Example {comp-val opt-opt} {comp-comp opt-val}
#            : args provides the values to analyze
#Results     : the comps
#%EndHeader######################################################################
proc extractCompArgs {syntax argval} {
    ## check how to handle components
    if {[lsearch $syntax "comp-comp"] >= 0} {
        set compSyntax 0
    } elseif {[lsearch $syntax "comp-val"] >= 0} {
        set compSyntax 1
    } else {
        error "extractCompArgs: Dont know how to handle components - missing comp-comp/comp-val in syntax '$syntax'"
    }
    ## check how to handle options
    if {[lsearch $syntax "opt-opt"] >= 0} {
        set optSyntax 0
    } elseif {[lsearch $syntax "opt-val"] >= 0} {
        set optSyntax 1
    } else {
        error "extractCompArgs: Dont know how to handle options - missing opt-opt/opt-val in syntax '$syntax'"
    }

    set components {}
    set options {}
    for {set argidx 0} {$argidx < [llength $argval]} {incr argidx} {
        if {[string range [lindex $argval $argidx] 0 0] == "-"} {
            if {$optSyntax} {
                lappend options [lindex $argval $argidx] [lindex $argval [incr argidx]]
            } else {
                lappend options [lindex $argval $argidx]
            }
        } else {
            if {$compSyntax} {
                lappend components [lindex $argval $argidx] [lindex $argval [incr argidx]]
            } else {
                lappend components [lindex $argval $argidx]
            }
        }
    }

    ## fini
    return [list -components $components -options $options]
}

#%ProcedureHeader################################################################
#Name        : checkCompAllowed
#
#Author      : Thomas Perschak 2004/01/07
#modified by : <none>
#Abstract    : Check the voptions array for components that are not allowed
#Arguments   : voptions array and the optionsAllowed
#Results     : error if fails
#%EndHeader######################################################################
proc checkCompAllowed {voptions optionsAllowed} {
    upvar $voptions options
    ## check for unallowed options
    if {[array names options] != $optionsAllowed} {
        ## list is different - search for unallowed options and make specific error
        foreach option [array names options] {
            if {[lsearch $optionsAllowed $option] == -1} {
                error "checkCompAllowed: Option '$option' not allowed"
            }
        }
    }
}

#%ProcedureHeader################################################################
#Name        : checkCompRange
#
#Author      : Thomas Perschak 2004/01/07
#modified by : <none>
#Abstract    : Check the voptions array for values out of range
#Arguments   : voptions array and the voptionsRange array
#Results     : error if fails
#%EndHeader######################################################################
proc checkCompRange {voptions voptionsRange} {
    upvar $voptions options
    upvar $voptionsRange optionsRange
    ## check options for unallowed values
    foreach option [array names options] {
        if {[array names optionsRange $option] == $option} {
            if {![checkRange $options($option) $optionsRange($option)]} {
                error "checkCompRange: Value '$options($option)' out of range for option '$option', possible range is defined as $optionsRange($option)"
            }
        }
    }
}
